#!/usr/bin/perl

use strict;
use warnings;
use Config4GNU::Sectioned;

package ApacheStyleParser::Section;
use base 'Config4GNU::Sectioned::Section';

sub createSection
{
	my $self = shift;
	my @words = @_;

    return ApacheStyleParser::Section->new(@words);
}

sub createProperty
{
	my $self = shift;
	my @words = @_;

	return ApacheStyleParser::Property->new(@words);
}

sub isSectionStart
{
	my $self = shift;
	my @words = @_;
	
	return scalar(@words) >= 3 &&
		$words[0]->word eq '<' &&
		substr($words[1]->word, 0, 1) ne '/';
}

sub isSectionEnd
{
	my $self = shift;
	my @words = @_;

	return scalar(@words) >= 3 &&
		$words[0]->word eq '<' &&
		substr($words[1]->word, 0, 1) eq '/';
}

package ApacheStyleParser;
use base ('Config4GNU::Sectioned', 'ApacheStyleParser::Section');

sub TokenizerClass
{
	return 'ApacheStyleParser::LineTokenizer';
}

package ApacheStyleParser::Property;
use base 'Config4GNU::Sectioned::Property';

package ApacheStyleParser::LineTokenizer;
use base 'Config4GNU::Sectioned::LineTokenizer';

# split string into words
# returns undef if no more words
sub nextToken
{
	my $self = shift;
	my $line = $self->{LINE};

	# return undef if no more words
	return undef if ($line eq '');
		
	my $curWord = '';
	my $inQuote = '';
	my $whitespace = '';
	
	while ($line =~ /^./)
	{
		my $c = $&;
		$line = $';
		
		if ($c eq '"' || $c eq "'")
		{
			if ($inQuote eq $c)
			{
				$inQuote = '';
			}
			elsif ($inQuote eq '')
			{
				$inQuote = $c;
			}
		}
		elsif ($c eq '\\' && length($line) >= 1)
		{
			# backslash... include the following character literally
			$c .= substr($line, 0, 1);
			$line = substr($line, 1);
		}
		elsif ($c =~ /\s/ && $inQuote eq '')
		{
			# found whitespace... suck up the rest of the whitespace,
			# and add this word to the resulting array
			$line =~ /^\s*/;
			$whitespace = $c . $&;
			$line = $';
			
			last;
		}
		elsif (index('<>', $c) != -1)
		{
			if ($curWord eq '')
			{
				$curWord = $c;
				last;
			}
			else
			{
				$line = $c . $line;
				last;
			}
		}
		$curWord .= $c;
	}
	$self->{LINE} = $line;
	return Config4GNU::Sectioned::Word->new($curWord, $whitespace);
}

1;
