#!/usr/bin/perl

# Copyright 2004 Jason Long

# This file is part of config4gnu-wbem.
#
# Config4gnu-wbem is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Config4gnu-wbem is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Config4gnu-wbem; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# USAGE:
#   Import the associated schema file (base.mof) into the OpenWBEM
#   CIMOM using owmofc.
#   Install this file to $(OPENWBEM_PREFIX)/lib/openwbem/perlproviders/

use strict;
use warnings;
use Config4GNU::WBEM;

use constant CLASSNAME_COMPUTERSYSTEM => 'Config4GNU_LinuxComputerSystem';

# initialize
ComputerSystem->register(CLASSNAME_COMPUTERSYSTEM);

#######################################################################
#######################################################################

package ComputerSystem;
use base 'Config4GNU::WBEM::SimpleProvider';

use constant CLASSNAME_COMPUTERSYSTEM => main::CLASSNAME_COMPUTERSYSTEM;
use constant STATUS_ENABLED => 2;

sub enumInstanceNames
{
	my $self = shift;
	my ($env, $ns, $className, $result, $cimClass) = @_;

	my $mcop = owperl::CIMObjectPath->new(CLASSNAME_COMPUTERSYSTEM, $ns);
	chomp (my $hostname = `hostname`);
	$mcop->addStringKeyValue('Name', $hostname);
	$mcop->addStringKeyValue('CreationClassName', CLASSNAME_COMPUTERSYSTEM);
	$result->handle($mcop);
}

sub getInstance
{
	my $provider = shift;
	my ($env, $ns, $instanceName, $localOnly, $includeQualifiers,
	  $includeClassOrigin, $propertyList, $cimClass) = @_;

	my $ci = $cimClass->newInstance;

	my $hostname = $instanceName->getStringKeyValue('Name');

	# fill in the key properties
	$ci->setStringProperty('Name', $hostname);
	$ci->setStringProperty('CreationClassName', CLASSNAME_COMPUTERSYSTEM);

	# other properties...
	$ci->setStringProperty('PrimaryOwnerName', 'root');
	$ci->setStringProperty('PrimaryOwnerContact', "root\@$hostname");
	$ci->setIntegerProperty('EnabledState', STATUS_ENABLED);
	$ci->setIntegerProperty('RequestedState', STATUS_ENABLED);
	$ci->setIntegerProperty('EnabledDefault', STATUS_ENABLED);

	$ci->setStringProperty('Caption', 'Computer System');
	$ci->setStringProperty('ElementName', $hostname);

	return $ci;
}
