#!/usr/bin/perl -w

# Copyright 2004 Jason Long

# This file is part of config4gnu-wbem.
#
# Config4gnu-wbem is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Config4gnu-wbem is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Config4gnu-wbem; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
package Config4GNU::Nagios::Status;

sub new
{
	my $class = shift;
	my $filename = shift;
	my $self = { 
		'FILENAME' => $filename };
	bless $self, $class;
	$self->init;
	return $self;
}

sub init
{
	my $self = shift;
	$self->{HOSTS} = [];
	$self->{SERVICES} = [];
	
	open INFILE, '<', $self->{FILENAME}
		or die;
	while (my $line = <INFILE>)
	{
		next if ($line =~ /^#/);
		chomp $line;
		my ($field1, $rest) = split(/;/, $line, 2);
		if ($field1 =~ /^\[(\d*)\] HOST$/)
		{
			my $hoststatus = Config4GNU::Nagios::Status::HostLine->new($1, $rest);
			push @{$self->{HOSTS}}, $hoststatus;
		}
		elsif ($field1 =~ /^\[(\d*)\] SERVICE$/)
		{
			my $servicestatus = Config4GNU::Nagios::Status::ServiceLine->new($1, $rest);
			push @{$self->{SERVICES}}, $servicestatus;
		}
		# ignore other lines
	}

	close INFILE;
}

package Config4GNU::Nagios::Status::Line;

sub new
{
	my $class = shift;
	my $timestamp = shift;
	my $unparsed_line = shift;
	
	my $self = {};
	bless $self, $class;
	$self->init($timestamp, $unparsed_line);
	return $self;
}

package Config4GNU::Nagios::Status::HostLine;
use base 'Config4GNU::Nagios::Status::Line';

sub init
{
	my $self = shift;
	my ($timestamp, $unparsed_line) = @_;
	
	my @fields = split(/;/, $unparsed_line, 20);
	$self->{FIELDS} = {
		'host_name' => $fields[0],
		'status' => $fields[1],
		'last_check_time' => $fields[2],
		'last_state_change' => $fields[3],
		'acknowledged' => $fields[4],
		'time_up' => $fields[5],
		'time_down' => $fields[6],
		'time_unreachable' => $fields[7],
		'last_notification_time' => $fields[8],
		'current_notification_number' => $fields[9],
		'notifications_enabled' => $fields[10],
		'event_handlers_enabled' => $fields[11],
		'checks_enabled' => $fields[12],
		'flap_detection_enabled' => $fields[13],
		'host_is_flapping' => $fields[14],
		'percent_state_change' => $fields[15],
		'scheduled_downtime_depth' => $fields[16],
		'failure_prediction_enabled' => $fields[17],
		'process_performance_data' => $fields[18],
		'plugin_output' => $fields[19],
		'last_update' => $timestamp };
}

package Config4GNU::Nagios::Status::ServiceLine;
use base 'Config4GNU::Nagios::Status::Line';

sub init
{
	my $self = shift;
	my ($timestamp, $unparsed_line) = @_;

	my @fields = split(/;/, $unparsed_line, 31);
	$self->{FIELDS} = {
		'host_name' => $fields[0],
		'service_description' => $fields[1],
		'status' => $fields[2],
		'retry_number' => $fields[3],
		'state_type' => $fields[4],
		'last_check_time' => $fields[5],
		'next_check_time' => $fields[6],
		'check_type' => $fields[7],
		'checks_enabled' => $fields[8],
		'accept_passive_checks' => $fields[9],
		'event_handlers_enabled' => $fields[10],
		'last_state_change' => $fields[11],
		'problem_acknowledged' => $fields[12],
		'last_hard_state' => $fields[13],
		'time_ok' => $fields[14],
		'time_unknown' => $fields[15],
		'time_warning' => $fields[16],
		'time_critical' => $fields[17],
		'last_notification_time' => $fields[18],
		'current_notification_number' => $fields[19],
		'notifications_enabled' => $fields[20],
		'latency' => $fields[21],
		'execution_time' => $fields[22],
		'flap_detection_enabled' => $fields[23],
		'service_is_flapping' => $fields[24],
		'percent_state_change' => $fields[25],
		'scheduled_downtime_depth' => $fields[26],
		'failure_prediction_enabled' => $fields[27],
		'process_performance_data' => $fields[28],
		'obsess_over_service' => $fields[29],
		'plugin_output' => $fields[30],
		'last_update' => $timestamp };
}

1;
