#!/usr/bin/perl -w

# Copyright 2004 Jason Long

# This file is part of config4gnu-wbem.
#
# Config4gnu-wbem is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# Config4gnu-wbem is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Config4gnu-wbem; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use Config4GNU::AbstractProvider;
use Config4GNU::AbstractProperty;
use strict;

package Config4GNU::SimpleKeyedValues;
use base 'Config4GNU::AbstractProvider';

sub init
{
    my $self = shift;

    open INFILE, '<', $self->{FILENAME}
            or throw Config4GNU::FileIOException errorMessage => "$!";
    my $comments = '';
    while (my $line = <INFILE>)
    {
		# TODO: handle line extenders (backslash at the end of the line)
		
        # check for a comment
        if ($line =~ /^\s*[#;]/)
        {
            # ignore the comment
            $comments .= $line;
        }
        # check for key=value
        elsif ($line =~ /^(\s*)([^=]+)=(.*)$/)
        {
            my $whitespace = $1;
			my $key = $2;
			chomp (my $value = $3);
			
            # create the backend object
			my $prop = $self->createProperty($key, $value);
    		$prop->{COMMENTS} = $comments;
			$prop->{whitespace} = $whitespace;

			# reset comments
			$comments = '';

            # add it to self
			$self->addPropertyObject($prop);
        }
        else
        {
			# unrecognized line, treat it as a comment
            $comments .= $line;
        }
    }
    close INFILE;

    $self->{END_COMMENTS} = $comments;
}

sub save
{
    my $self = shift;

	open OUTFILE, '>', $self->{FILENAME}
			or throw Config4GNU::FileIOException errorMessage => "$!";

    my @properties = @{$self->{PROPERTIES}};
    for my $prop (@properties)
    {
        $prop->writeConfig(\*OUTFILE);
    }
    print OUTFILE $self->{END_COMMENTS};
	
	close OUTFILE;
}

sub createProperty
{
	my $self = shift;
	my ($prop_name, $prop_value) = @_;

    return Config4GNU::SimpleKeyedValues::Property->new(
			$prop_name, $prop_value);
}

package Config4GNU::SimpleKeyedValues::Property;
use base 'Config4GNU::AbstractProperty';

sub writeConfig
{
    my $self = shift;
	my $fh = shift;
		
	# sanity check parts of each property
	# TODO: check comments
	#		for each line, should begin with valid comment character
	#		should end with a linefeed
	if (!defined($self->{COMMENTS}))
	{
		$self->{COMMENTS} = '';
	}
	if (!defined($self->{whitespace}) || $self->{whitespace} =~ /\S/)
	{
		# whitespace contains a nonwhitespace character, so reset it
		$self->{whitespace} = '';
	}

	# output the property
    print $fh $self->{COMMENTS};
    print $fh $self->{whitespace}, $self->{NAME}, '=', $self->{VALUE}, "\n";
}

1;
